# sb::Driver::Mysql
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
# [note] sb::Driver::Mysql requires DBD::mysql
package sb::Driver::Mysql;
use strict;
use sb::Language ();
use sb::Driver ();
use sb::Driver::SqlBase (); # included 'use DBI'
use base qw( sb::Driver::SqlBase );
use vars qw( $VERSION );
$VERSION = '0.11';
# 0.11 [2008/11/24] modified _connect to use sb->debug
# 0.10 [2008/11/05] modified _connect/convert_type to disable auto increment
# 0.09 [2007/12/03] modified _connect to fix a bug
# 0.08 [2007/08/11] added _set_names to set character set correctly
# 0.07 [2007/08/06] changed _connect
# 0.06 [2007/07/16] changed return to make them consistent
# 0.05 [2007/07/09] changed convert_type to call base method with correct params
# 0.04 [2006/03/31] added driver_param and table_param
# 0.03 [2006/03/15] changed _connect to set character code correctly
# 0.02 [2006/03/04] moved basic implementations into SqlBase.pm
# 0.01 [2006/03/03] the first implementation
# 0.00 [2006/02/01] generated
# --------------------------------------
# // defines / static variables
sub DEFALUT_NAME     (){ 'sb' };
sub DEFAULT_HOST     (){ '127.0.0.1' };
sub DEFAULT_PORT     (){ 3306 };
sub DEAFULT_SOCKET   (){ '' };
sub DEFALUT_USER     (){ 'root' };
sub DEFALUT_PASSWORD (){ '' };
# --------------------------------------
# // override base class
sub driver_param
{
	my $class = shift;
	return (
		'name' => DEFALUT_NAME,
		'host' => DEFAULT_HOST,
		'port' => DEFAULT_PORT,
		'sock' => DEAFULT_SOCKET,
		'user' => DEFALUT_USER,
		'pass' => DEFALUT_PASSWORD,
	);
}
sub table_param
{
	my $class = shift;
	return (
		'id'   => 'INTEGER',
		'str'  => 'VARCHAR',
		'text' => 'TEXT',
		'date' => 'INTEGER',
		'zone' => 'VARCHAR(7)',
		'int'  => 'INTEGER', 
		'list' => 'TEXT',
		'file' => 'VARCHAR(64)',
	);
}
# --------------------------------------
# // override base class
sub setup
{
	my $self = shift;
	$self->SUPER::setup(
		$self->driver_param,
		'option' => { # options - use for DBI
			'PrintError' => 0,
			'RaiseError' => 0,
		},
		@_
	);
	$self->dbh( $self->_connect() );
	$self->_set_names();
	$self->load_ids();
	return $self;
}
# --------------------------------------
# // public functions - class method
sub convert_type
{
	my $class = shift;
	my $name = shift;
	my $type = $class->SUPER::convert_type($name);
	$type = 'VARCHAR(255)' if ($type eq 'UNKNOWN');
	$type .= ' NOT NULL PRIMARY KEY' if ($name =~ /^UNI/i);
	return $type;
}
# --------------------------------------
# // private functions - instance method
sub _connect
{
	my $self = shift;
	my $handle = undef;
	my $source = 'DBI:mysql:database=' . $self->{'name'};
	$source .= ';hostname=' . $self->{'host'} if ($self->{'host'});
	$source .= ';port=' . $self->{'port'} if ($self->{'port'});
	$source .= ';mysql_socket=' . $self->{'sock'} if ($self->{'sock'});
	$handle = DBI->connect($source,$self->{'user'},$self->{'pass'},$self->{'option'});
	sb->debug(sb->string('Failed to connect database.') . ' ' . $DBI::errstr) if (!$handle);
	# $handle->do('SET @@sql_mode=\'NO_AUTO_VALUE_ON_ZERO\';');
	return $handle;
}
sub _set_names
{
	my $self = shift;
	return if (! $self->dbh);
	my $sth = $self->dbh->prepare('show variables like "character_set_database"') or return;
	$sth->execute() or return;
	my $charset = $sth->fetchall_hashref('Variable_name')->{'character_set_database'}{'Value'};
	if (defined($charset) and $charset ne 'latin1')
	{
		my $code = (sb::Language->get->charcode eq 'euc')  ? 'ujis' # euc-jp must indicate as 'ujis'
		         : (sb::Language->get->charcode eq 'sjis') ? 'sjis' # shift-jis
		         : 'utf8'; # otherwise, set as utf8
		$self->dbh->do("SET NAMES '$code';");
	}
	return;
}
1;
__END__
