/*!
  spica javascript libraries - listtable.js
  == written by Takuya Otani <takuya.otani@gmail.com> ===
  == Copyright (C) 2006 SimpleBoxes/SerendipityNZ Ltd. ==
*/

// [guideline]
// -- http://serennz.sakura.ne.jp/guidelines/js_en_000.txt
// -- http://serennz.sakura.ne.jp/guidelines/js_ja_000.txt

// ver 0.00 [2008/11/01] the first attempt

Spica.Library.require('element','adminutils');
Spica.ListTable = function(table)
{
	this.table = table;
	this.status = Spica.$('showSelected');
	this.selectall = Spica.getElementsByClassName('selectAll',table);
	this.selecttype = Spica.$('selectType');
	this.submit = Spica.$('submit');
	this.row_of = {};
	this.total = 0;
	return this;
};
Spica.ListTable.prototype = {
	initialize : function()
	{
		var self = this;
		if (!self.table) return;
		var rows = self.table.getElementsByTagName('tr');
		// --- check through rows of table
		for (var i=0,n=rows.length;i<n;i++)
		{
			var row = rows[i];
			if (!row.className.match(/odd$/) && !row.className.match(/even$/)) continue;
			// --- updates elements to set title attribute
			var cols = row.getElementsByTagName('td');
			for (var j=0,m=cols.length;j<m;j++)
			{
				var col = cols[j].firstChild;
				if (cols[j].className != 'elem') continue;
				if (!col || !col.innerHTML) continue;
				if (!col.getAttribute('title')) col.setAttribute('title',col.innerHTML);
			}
			// --- gets a checkbox to select the row
			var check = row.getElementsByTagName('input');
			var sel = null;
			for (var j=0,m=check.length;j<m;j++)
			{
				if (check[j].getAttribute('name') != 'sel') continue;
				sel = check[j];
				break;
			}
			if (!sel) continue;
			self.row_of[sel.value] = { obj: row, sel: sel };
			self.total++;
			self.updateRow(sel.value);
			Spica.Event.register(sel,'click',self.toggleRow(sel.value,true));
			Spica.Event.register(cols[0],'click',self.toggleRow(sel.value,false));
			cols[0].style.cursor = 'pointer';
		}
		self.updateStatus();
		// --- register event handler
		Spica.Event.register(self.selecttype,'change',self.selectType());
		for (var i=0,n=self.selectall.length;i<n;i++)
			Spica.Event.register(self.selectall[i],'click',self.toggleAll(i));
		Spica.Event.register(self.submit,'click',self.checkAction());
	},
	toggleRow : function(id,direct)
	{
		var self = this;
		return function() {
			if (!self.row_of[id].obj || !self.row_of[id].sel) return;
			if (!direct) self.row_of[id].sel.checked = !self.row_of[id].sel.checked;
			self.updateRow(id);
			self.updateStatus();
		};
	},
	toggleAll : function(idx)
	{
		var self = this;
		return function() {
			(self.selectall[idx].checked) ? self.selectAll() : self.releaseAll();
		};
	},
	selectType : function()
	{
		var self = this;
		return function() {
			var selected = self.selecttype.value;
			if (!selected) return;
			for (var id in self.row_of)
			{
				var obj = Spica.DOM.getParent(self.row_of[id].sel);
				if (!obj) continue;
				self.row_of[id].sel.checked = (obj.className.indexOf(selected) > -1);
				self.updateRow(id);
			}
			self.updateStatus();
		}
	},
	selectAll : function()
	{
		var self = this;
		for (var id in self.row_of)
		{
			self.row_of[id].sel.checked = true;
			self.updateRow(id);
		}
		self.updateStatus();
	},
	releaseAll : function()
	{
		var self = this;
		for (var id in self.row_of)
		{
			self.row_of[id].sel.checked = false;
			self.updateRow(id);
		}
		self.updateStatus();
	},
	updateRow : function(id)
	{
		var self = this;
		var name = (self.row_of[id].obj.className.match(/odd$/)) ? 'odd' : 'even';
		if (self.row_of[id].sel.checked) name = 'selected_' + name;
		self.row_of[id].obj.className = name;
	},
	getSelected : function()
	{
		var self = this;
		var selected = 0;
		for (var id in self.row_of)
		{
			if (self.row_of[id].sel.checked) selected++;
		}
		var labels = ['mid','bid','iid','lid','tid','pid','aid','cid'];
		for (var i=0,n=labels.length;i<n;i++)
		{
			if (selected) break;
			var check = Spica.$(labels[i]);
			if (!check || typeof(check.value) == 'undefied') continue;
			if (check.value != null) selected++;
		}
		return selected;
	},
	updateStatus : function()
	{
		var self = this;
		if (!self.status) return;
		var selected = self.getSelected();
		self.status.innerHTML = selected + '/' + self.total;
		for (var i=0,n=self.selectall.length;i<n;i++)
			self.selectall[i].checked = (selected == self.total);
	},
	checkAction : function()
	{
		var self = this;
		return function(evnt) {
			evnt = (evnt) ? evnt : ((window.event) ? event : null);
			var selected = self.getSelected();
			if (selected == 0)
			{
				alert(Spica.$_('No item is selected.'));
				return Spica.Event.stop(evnt);
			}
			var action = Spica.$('regi_action');
			if (action && action.value == '')
			{
				alert(Spica.$_('No action is selected.'));
				return Spica.Event.stop(evnt);
			}
			if (!confirm(Spica.$_('Are you sure you want to execute?')))
				return Spica.Event.stop(evnt);
			return true;
		};
	}
};
